"use client";

import { deleteAssignment } from "@/app/lib/assignment/action";
import AddButton from "@/components/buttons/add";
import DataTable from "@/components/tables/DataTable";
import { DeleteIcon, EditIcon } from "@chakra-ui/icons";
import {
  Button,
  Flex,
  Icon,
  useColorModeValue,
  useToast,
} from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { columnsDataAssignments } from "../variables/columns";
import { hasPermission } from "@/utils/navigation";
import { useGeneral } from "@/contexts/GeneralContext";
import { TableData } from "@/variables/columnsData";
import { extractPagination } from "@/utils/common";
import Filters from "@/components/filters/Filters";
import { MultiSelectOption } from "@/components/fields/MultiSelect";

interface IAssignmentsTableProps {
  data: { data: TableData[] };
  users: MultiSelectOption[];
  ibcs: MultiSelectOption[];
  batches: MultiSelectOption[];
}

const AssignmentsTable: React.FC<IAssignmentsTableProps> = ({ data, users, ibcs, batches }) => {
  const toast = useToast();
  const router = useRouter();
  const brandColor = useColorModeValue("brand.500", "white");
  const { currentUser } = useGeneral();

  const paginationData = extractPagination(data);

  const deleteUser = async (id: number) => {
    const res = await deleteAssignment(id);
    if (res?.status == 200) {
      toast({
        title: "Success",
        description: res.message,
        status: "success",
        position: "top-right",
        duration: 2000,
        isClosable: true,
      });
      router.refresh();
    } else {
      toast({
        title: "Error",
        description: res.message,
        status: "error",
        position: "top-right",
        duration: 9000,
        isClosable: true,
      });
    }
  };

  const handleAddClick = () => {
    router.push("/admin/assignments/create");
  };

  const handleEditClick = (id: number) => {
    router.push(`/admin/assignments/edit/${id}`);
  };

  const action = (user: number) => (
    <Flex gap={2}>
      {hasPermission(currentUser.permissions, "assignment", "edit") && (
        <Button
          variant="blank"
          alignItems="center"
          justifyContent="center"
          w="24px"
          h="24px"
          lineHeight="100%"
          borderRadius="10px"
          minW={0}
          onClick={() => handleEditClick(user)}
          isDisabled={
            !hasPermission(currentUser.permissions, "assignment", "edit")
          }
        >
          <Icon as={EditIcon} color={brandColor} w="20px" h="20px" />
        </Button>
      )}
      {hasPermission(currentUser.permissions, "assignment", "delete") && (
        <Button
          variant="blank"
          alignItems="center"
          justifyContent="center"
          w="24px"
          h="24px"
          lineHeight="100%"
          borderRadius="10px"
          minW={0}
          onClick={() => deleteUser(user)}
          isDisabled={
            !hasPermission(currentUser.permissions, "assignment", "delete")
          }
        >
          <Icon as={DeleteIcon} color={brandColor} w="20px" h="20px" />
        </Button>
      )}
    </Flex>
  );

  return (
    <>
      <DataTable
        addButton={hasPermission(currentUser.permissions, "assignment", "create") && <AddButton onClick={handleAddClick} />}
        columnsData={columnsDataAssignments}
        tableData={data?.data as TableData[]}
        paginate={paginationData}
        isSelectable={false}
        showPageSize={false}
        showSearch={false}
        actionSlot={(user: number) => action(user)}
        filterSlot={
          <Filters users={users} ibcs={ibcs} batches={batches} showExport={false} showSearch={false} attempt={true} />
        }
      />
    </>
  );
};

export default AssignmentsTable;
